function FITTING_SPLINES(inputfilename,outfilename,pauselen,plotall)



% The tracks.<date>.txt file is the output of the tracking component.



% It defines a structure with each field representing a single trajectory.

% A trajectory is a two dimensional array (an array of points).

% (note: ideally, this output should be a .mat file instead )

% Auther : Gurkirt Singh

% Date   : 15 April 2010 



% changing  Input file name to tmp.m%

system(['cp ',inputfilename,' tmp.m']);

tmp



% Get the 'references' to all the trajectories

% (here the structure name is <code>TRACKS</code>).

fn = fieldnames(TRACK);


rec=cell(length(fn),1);

% Number of control points of each spline.

controlPointsNumber = 6;

%the elements along the dimension of A specified by a scalar



% Degree of spline basis function (3: cubic).

degree = 3;



% Minimal distance between two consecutive points.

% (points which are too close will be removed)

dist = 7;



% The output 2-dimensional array of control points for all trajectories.

controlPoints=zeros(size(fn,1),12);



% writing the Intial information%

fid=fopen(outfilename,'wt');

fprintf(fid,'%% Total number of trajectories in file are  %d \n',size(fn,1));

fprintf(fid,'%% X and Y are normalized by dividing 640 and 460 respectively\n');

fprintf(fid,'%% Image size is 640*460\n\n');





for n = 1 : length(fn)

    

    % Show progres.

    %[n; length(fn)];

    

    % Prepare the data.

    [x y] = transformForSplines(TRACK.(fn{n}), 640, 460, dist);

    [len, coords] = size(y);

    

    if len > controlPointsNumber

        

        % Fit a spline to the trajectory.

        [xcp,ycp] = splinefit(x, y, controlPointsNumber, degree);

        controlPoints(n,:) = [xcp', ycp'];

        

        % Reconstruct the spline given its control points.

        [xr,yr] = splinerec(xcp,ycp,len,degree);

        
        rec{n,1}=[xr yr];
        % deviation will store the stamdard deviation for each trajectory

        % every time whicgh is written on file

        deviation=0;

        z=[x y]-[xr yr];

        for i=1:len

            deviation=deviation+(norm(z(i,:)))^2;

        end

        deviation=sqrt(deviation/len);

        % printing deviation to   file

        fprintf(fid,'Deviation.R%d=[ %f ];\n',n,deviation);

        

        fprintf(fid,'Controlpoints.R%d=[',n);

        

        % printing control points

        

        for m=1:controlPointsNumber-1

            fprintf(fid,'[%6.4f %6.4f];',xcp(m),ycp(m));

        end

        fprintf(fid,'[%6.4f %6.4f]];\n\n',xcp(end),ycp(end));

        

        %%% Uncomment to see the plot of each trajectory %%%

      if pauselen > 0

        close all;

        open('small.fig');

        hold on

        plot(round(640*x), round(y*460), 'b-');

        hold on;

        plot(round(640*xcp),round(460*ycp),'k:o');

        hold on;        

        plot(round(640*xr),round(460*yr),' r-');

        pause(pauselen);

      end

    end

end

fclose(fid);
if plotall==1
open('small.fig');
hold on
colors = ['rgbcmyk'];
for n=1:size(rec,1);
[nn,mm] = size(colors);
colorNo = mod(n-1, mm) + 1;
traj=rec{n,1};
plot(round(traj(:,1)*640),round(traj(:,2)*460),[colors(colorNo),'-']);
hold on
end
end


